@@ -52,7 +52,7 @@ package App::wsgetmail;
 
 use Moo;
 
-our $VERSION = '0.08';
+our $VERSION = '0.10';
 
 =head1 NAME
 
@@ -74,6 +74,8 @@ where C<wsgetmail.json> looks like:
     "username": "rt-comment@example.com",
     "folder": "Inbox",
     "stripcr": 0,
+    "size_limit": 10485760,
+    "body_size_limit": 1048576,
     "command": "/opt/rt5/bin/rt-mailgate",
     "command_args": "--url=http://rt.example.com/ --queue=General --action=comment",
     "command_timeout": 30,
@@ -206,10 +208,14 @@ sub process_message {
     my ($self, $message) = @_;
     my $client = $self->client;
     my $filename = $client->get_message_mime_content($message->id);
-    unless ($filename) {
+    if (not defined $filename) {
         warn "failed to get mime content for message ". $message->id;
         return 0;
     }
+    elsif (not $filename) {
+        # failure, but handled silently
+        return 1;
+    }
     my $ok = $self->forward($message, $filename);
     if ($ok) {
         $ok = $self->post_fetch_action($message);
@@ -323,22 +329,14 @@ L<"Configure a client application to access a web API"
 quickstart|https://docs.microsoft.com/en-us/azure/active-directory/develop/quickstart-configure-app-access-web-apis#application-permission-to-microsoft-graph>,
 under the section "Add permissions to access Microsoft Graph." When selecting
 the type of permissions, select "Application permissions." When prompted to
-select permissions, select all of the following:
+select permissions, select the following items:
 
 =over 4
 
 =item * Mail.Read
 
-=item * Mail.Read.Shared
-
 =item * Mail.ReadWrite
 
-=item * Mail.ReadWrite.Shared
-
-=item * openid
-
-=item * User.Read
-
 =back
 
 =back
@@ -424,6 +422,25 @@ but you need to enable it explicitly in your configuration.
 This option is helpful if you are forwarding email to a Linux
 utility that doesn't work with CRLF line-endings.
 
+=item size_limit
+
+Set this to the max size in bytes. Messages bigger than it will be skipped.
+Absence or 0 means to not limit size.
+
+E.g. to skip messages bigger than C<10MiB>, you can set it to C<10485760>.
+
+=item body_size_limit
+
+Set this to the max body size in bytes. Messages with body bigger than it will
+be skipped. Absence or 0 means to not limit body size.
+
+E.g. to skip messages with body bigger than C<1MiB>, you can set it to
+C<1048576>.
+
+The difference between C<size_limit> and C<body_size_limit> is the former
+limits the size of the whole message, while the latter parses messages, skips
+attachments and only checks text/plain and text/html parts.
+
 =item command
 
 Set this to an executable command. You can specify an absolute path,
@@ -459,6 +476,19 @@ Set this to 1 to preserve the temporary files after processing.
 
 When C<"debug"> is also set the filenames will be reported on STDERR.
 
+=item debug
+
+Set this to enable additional diagnostic and status messages.
+
+=item quiet
+
+Set this to put wsgetmail into C<quiet> mode. This mode intended for use in cron
+or other automation.
+
+When in C<quiet> mode wsgetmail produces no output unless there is an error, in
+which case the configuration filename and error messages will be printed to
+STDERR.
+
 =back
 
 =head1 TESTING AND DEPLOYMENT
@@ -516,8 +546,22 @@ configuration, just run wsgetmail with different configurations:
 
 Microsoft applies some limits to the amount of API requests allowed as
 documented in their L<Microsoft Graph throttling guidance|https://docs.microsoft.com/en-us/graph/throttling>.
-If you reach a limit, requests to the API will start failing for a period
-of time.
+If you reach a limit, requests to the API will start failing with HTTP
+status code 429 for a period of time.
+
+=head2 Office 365 API Errors
+
+Best Practical Solutions has observed that very rarely the Microsoft Graph API
+will return a 5xx error code when attempting to list mail folder details,
+retrieve the message list, fetch message details, or delete a message. The
+cause for this is currently unknown, except that it is not related to the API
+limits mentioned above.
+
+Despite the relative rarity, if you run wsgetmail often enough you can see this
+multiple times in a typical day.
+
+Therefore, by default wsgetmail now ignores C<5xx> response codes on all API
+calls, instead treating them as logically empty success response.
 
 =head1 SEE ALSO
 
@@ -539,7 +583,7 @@ Best Practical Solutions, LLC <modules@bestpractical.com>
 
 =head1 LICENSE AND COPYRIGHT
 
-This software is Copyright (c) 2015-2022 by Best Practical Solutions, LLC.
+This software is Copyright (c) 2015-2026 by Best Practical Solutions, LLC.
 
 This is free software, licensed under:
 
